//*******************************************************
//
//	Raspberry Pi Power Lock
//
//	ATtiny13
//		hfuse:0xFB	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2015.1.15	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/eeprom.h>

#include	"main.h"

volatile uchar	WTimer;
volatile uchar	Timer;
volatile uchar	SwCountDS;
volatile uchar	SwCountSec;

//*******************************************************
//	タイマー０比較一致割り込み
//*******************************************************
ISR(TIM0_COMPA_vect){

	if (WTimer)	--WTimer;
	if (Timer)	--Timer;

	// スイッチカウント
	if (IS_SW_ON){
		if (++SwCountDS >= 100){
			SwCountDS = 0;
			if (SwCountSec < 255){
				SwCountSec++;
			}
		}
	} else {
		SwCountDS = 0;
		SwCountSec = 0;
	}
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main (void) {
	uchar	last_ctrl = 1;
	uchar	shutdown_count;

	cli();	// 割り込み禁止

	PORTB = (1<<SWITCH)|(1<<CTRL_IN);
	DDRB = (1<<PLOCK)|(1<<LED);

	// スリープ動作設定
	MCUCR = (1<<SE);	// スリープ許可, アイドルモード

	// タイマ0設定
	TCCR0A = (1<<WGM01);			// CTCモード
	TCCR0B = (1<<CS01)|(1<<CS00);	// プリスケーラ 1/64
	OCR0A = F_CPU/64/100 - 1;		// 100Hz
	TIMSK0 = (1<<OCIE0A);	// 比較A割り込み許可

	ACSR |= (1<<ACD);	// アナログコンパレータ禁止

	sei();

	wait_timer(10);		// 100ms

	POWER_LOCK;
	LED_ON;

	// スイッチが放されるまで待つ
	SLEEP;
	while(IS_SW_ON){
		WDR;
		SLEEP;
	}

	// メインループ
	while (SwCountSec < 10){
		uchar	ctrl;

		// 通常動作
		LED_ON;
		while (SwCountSec < 10){
			WDR;
			SLEEP;

			ctrl = IS_CTRL;
			if (!last_ctrl && ctrl)	break;	// CTRL_IN 立ち上がり

			last_ctrl = ctrl;
		}

		shutdown_count = 0;

		while (SwCountSec < 10){
			WDR;
			SLEEP;

			if (SwCountSec >= 1)	break;

			if (!Timer){
				Timer = 50;
				LED_FLASH;
				shutdown_count++;
			}
			if (shutdown_count >= 24)	break;

			ctrl = IS_CTRL;
			if (!last_ctrl && ctrl){	// CTRL_IN 立ち上がり
				shutdown_count = 0;
			}
			last_ctrl = ctrl;
		}

		if (shutdown_count >= 24)	break;
	}

	POWER_OFF;
	LED_OFF;
	for (;;){
		WDR;
		SLEEP;
	}
}
